Attribute VB_Name = "odbctest"
'*********************************************************************
'/* COPYRIGHTS:                                                      */
'/*                                                                  */
'/*   This module contains code made available by IBM Corporation on */
'/* an AS IS basis.  Any one receiving the module is  considered  to */
'/* be  licensed under IBM copyrights to use the IBM-provided source */
'/* code in any way he or  she  deems  fit,  including  copying  it, */
'/* compiling  it,  modifying  it,  and  redistributing  it, with or */
'/* without modifications.  No license  under  any  IBM  patents  or */
'/* patent  applications  is  to  be  implied  from  this  copyright */
'/* license.                                                         */
'/*                                                                  */
'/*   A user of the module should understand that IBM cannot provide */
'/* technical support for the module and will not be responsible for */
'/* any consequences of use of the program.                          */
'/*                                                                  */
'/*   Any notice,  including this one, are not to  be  removed  from */
'/* the module with out the prior written consent of IBM.            */
'/*                                                                  */
'*********************************************************************

Option Explicit
'This module contains global variables, public functions
'and Windows API declares.


'Globals...
Global ghEnv As Long
Global ghDbc As Long                'Only one connection per app
Global gDSN As String               'Active DSN name
Global Const gAppName = "ODBC Test" 'Dialog box titles
Global rc As Long                   'ODBC api rc

'Data is fetched using two public functions BindColAll and GetDataAll
'They use the following global variables
Global Const MAXCOLSTRING = 50      'Max length of SQL_C_STRING formated field in RS
Global Const MAXNUMCOL = 300        'Maximum number of columns in RS
Global gsColName() As String        'Column names of result set
Type tResultSetData
   szCharValue() As Byte            'Actual data of field
   lCharLen As Long                 'String length
End Type
Global gRSData() As tResultSetData  'One Row of C_String formated fields

'Global for block insert
Global Const BLKSIZE = 30           'Number of rows to insert in one block

'Structure that matches QCUSTCDT
Type cQCUSTCDT
   CUSNUM As Long
   LstNam(8) As Byte
   INIT(2) As Byte      '   As String * 3
   Street(12) As Byte   ' As String * 13
   City(5) As Byte      '   As String * 6
   State(1) As Byte     '   As String * 2
   ZIPCOD(4) As Byte    '  As Integer
   CDTLMT As Single
   CHGCOD  As Integer
   BALDUE As Single
   CDTDUE As Single
End Type


'Globals used with form frmProcCol to retrieve Catalog info
Type tCatalogInfo
   LibName As String
   ProcName As String
   Option As String
End Type
Global CatalogReq As tCatalogInfo


'Declares for Windows APIs
Public Declare Function CharNext Lib "user32" Alias "CharNextA" _
                           (ByRef lpsz As Byte) As Long
Public Declare Function CharPrev Lib "user32" Alias "CharPrevA" _
                           (ByRef lpszStart As Byte, ByRef lpsz As Byte) As Long
Public Declare Function VarPtr Lib "vb40032.dll" (ByRef Ptr() As Any) As Long

'Public Declare Function SetWindowPos Lib "user32" _
'               (ByVal h%, ByVal hb%, ByVal X%, ByVal _
'                  Y%, ByVal cx%, ByVal cy%, ByVal f%) As Integer

Public Declare Function ShellExecute Lib "shell32.dll" Alias _
                     "ShellExecuteA" (ByVal hwnd As Long, ByVal lpOperation As _
                     String, ByVal lpFile As String, ByVal lpParameters As String, _
                     ByVal lpDirectory As String, ByVal nShowCmd As Long) As Long
Public Declare Function GetDesktopWindow Lib "user32" () As Long

Declare Sub Sleep Lib "kernel32" (ByVal dwMilliseconds As Long)
Public Function RmtCmd(Cmd As String) As Integer
   '*************************************************
   'Run a CL Command via ODBC stored procedure call
   '
   'Globals:
   '     ghDbc - ODBC connection handle
   'Output:
   '      rc from the execdirect
   '******************************************************
   Dim S As String
   Dim hStmt As Long

   rc = SQLAllocStmt(ghDbc, hStmt)
   If (rc <> SQL_SUCCESS And rc <> SQL_SUCCESS_WITH_INFO) Then
      'About the only errors possible are out of memory or invalid hDbc
      PrintError ghDbc, 0
      RmtCmd = rc
      Exit Function
   End If

   S = "CALL QSYS.QCMDEXC('" & UCase(Cmd) & "',"
   S = S & Format(Len(Cmd), "0000000000.00000")
   S = S & ")"

   Screen.MousePointer = vbHourglass
   rc = SQLExecDirect(hStmt, S, SQL_NTS)
   If (rc = SQL_ERROR) Then
      PrintError ghDbc, hStmt
   End If

   Call SQLFreeStmt(hStmt, SQL_DROP)
   Screen.MousePointer = 0
      
   RmtCmd = rc
   
End Function



Public Function BindColAll(hStmt) As Boolean
'***********************************************************************
' This function will determine the number of columns in a result set and
' bind all of the columns to a global user defined type that will
' contain the data (formatted for SQL_C_STRING) and the length of the
' data.
' If an error occurs it returns FALSE.  It is up to the caller to
' retrieve and handle the ODBC error.
'
'GLOBALS:
'  MAXCOLSTRING   - Maximum length of a column name
'  MAXNUMCOL      - Maximum # of columns
'  gsColName     - Array holding all the column names
'  gRSData       - UDT holding the column value and string length
'***********************************************************************

   Dim S As String
   Dim icol As Integer
   
   'NumResultCols
   Dim cbColCount As Integer
   
   '//ColAttributes
   Dim sColName As String           'Name of column   MAXCOLSTRING
   Dim szColName() As Byte
   Dim pcbColName As Integer        'Returned length of col name
   Dim pfSqlType As Integer         'SQL type of col
   Dim pcbColDef As Long            'Precision
   Dim pibScale As Integer          'Scale
   Dim pfNullable As Integer        'Nullable

   '//BindCol
   Dim cbValueMax As Long
   
   '*************** Find Number of Result Cols **********************
   cbColCount = 0
   rc = SQLNumResultCols(hStmt, cbColCount)
   If (Not (rc = SQL_SUCCESS Or rc = SQL_SUCCESS_WITH_INFO)) Then
      BindColAll = False
      Exit Function
   End If
   'Stored Procedures can return rc=0 but no result set so
   'guard against bug of calling BindColAll when no RS is available.
   If cbColCount = 0 Then
      BindColAll = True
      Exit Function
   End If
   
   '//set size of data fields
   'ReDim gsCharValue(cbColCount) As String
   'ReDim gcbValue(cbColCount) As Long
   ReDim gsColName(cbColCount - 1) As String
   ReDim gRSData(cbColCount - 1) As tResultSetData
   'Size the byte array to hold the data (should actually use display size)
   For icol = 0 To cbColCount - 1
      ReDim gRSData(icol).szCharValue(MAXCOLSTRING - 1) As Byte
   Next icol
   
   '************ Describe and Bind the Columns *******************

   sColName = Space(MAXCOLSTRING)
   For icol = 1 To cbColCount
      If icol = (MAXNUMCOL + 1) Then Exit For      'Max number of cols?
         
      rc = SQLDescribeCol(hStmt, icol, _
                           sColName, _
                           Len(sColName), _
                           pcbColName, _
                           pfSqlType, _
                           pcbColDef, _
                           pibScale, _
                           pfNullable)
      If (Not (rc = SQL_SUCCESS Or rc = SQL_SUCCESS_WITH_INFO)) Then
         BindColAll = False
         Exit Function
      End If
      '//Save the column name in global array
      gsColName(icol - 1) = Left(sColName, pcbColName)
      S = gsColName(icol - 1)
      
      '// Bind the data to the global array
      cbValueMax = UBound(gRSData(icol - 1).szCharValue) + 1 '//max size must be less then array
      rc = SQLBindColc(hStmt, icol, SQL_C_CHAR, _
                       gRSData(icol - 1).szCharValue(0), _
                       cbValueMax, _
                       gRSData(icol - 1).lCharLen)
      If (Not (rc = SQL_SUCCESS Or rc = SQL_SUCCESS_WITH_INFO)) Then
         BindColAll = False
         Exit Function
      End If
      
   Next icol
   
   BindColAll = True
End Function


Public Sub DisplayURL(URL As String)
'Rather then search for the installed browser, this launches
'the default browser associated with URL links.
'Both Netscape and IE like to register themselves for .URL files

   #If Win32 Then
      Dim rc As Long
      Dim hwndApp As Long
   #ElseIf Win16 Then
      Dim rc As Integer
      Dim hwndApp As Integer
   #End If

'On Error Resume Next
Const SW_SHOWNORMAL = 1
Dim I As Integer


   hwndApp = GetDesktopWindow()
   'ERROR_NO_ASSOCIATION  31 win16 and win32
   'For Win 16, other error values are:
   'Error Meaning 0  System was out of memory or executable file was corrupt.
   '2  The file was not found.
   '3  The path was not found.
   '5  Attempt was made to link to a task dynamically, or there was a sharing
    '6  Library required separate data segments for each task.
   '8  There was insufficient memory to start the application.
   '10 The Windows version was incorrect.
   '11 The executable file was invalid. Either it was not a Windows-based
   '12 Application was designed for a different operating system.
   '13 Application was designed for MS-DOS version 4.0.
   '14 Type of executable file was unknown.
   '20 Dynamic-link library (DLL) file was invalid. One of the DLLs required to
   '21 Application requires Microsoft Windows 32-bit extensions.
   
   'For Win 32, the error values are
   'Error Meaning 0  System was out of memory or executable file was corrupt.
   '2  The specified file was not found.
   '3  The specified path was not found.
   '11 The .EXE file is invalid (non-Win32 .EXE or error in the .EXE image)
   '26 A sharing violation occurred.
   '27 The filename association is incomplete or invalid.
   '31 There is no application associated with the given filename extension.
   
   rc = ShellExecute(hwndApp, "Open", URL, "", "c:\", _
                          SW_SHOWNORMAL%)
   Select Case rc
   Case 0
      MsgBox "Not enough memory to load web browser", 0, gAppName
   Case 2
      MsgBox "Unable to locate URL shortcut", 0, gAppName
   Case 31
      MsgBox "Unable to locate web browser.", 0, gAppName
   Case Is > 32   ' Valid handle so it worked
   Case Else   ' Other values
      MsgBox "Error attempting to start web browser.  Return Code: " & Str(rc), _
               0, gAppName
   End Select
   
End Sub

Public Function GetDataAll(hStmt) As Boolean
'************************************************************************************
'   FUNCTION:   GetDataAll
'
'   PURPOSE:    Fetch all rows in result set into the address used in BindColAll
'               Add each row to the List View control on form frmMain.
'               Update the text box on fmrMain with the number of rows returned.
'               If an error occurs it returns FALSE.  It is up to the caller to
'               retrieve and handle the ODBC error.
'
'   CALLS:      BindColAll
'   GLOBALS:
'               gsColName     - Array holding all the column names
'               gRSData       - UDT holding the column value and string length
'*************************************************************************************/

   Dim I As Integer, j As Integer, K As Integer
   Dim S As String, S1 As String
   Dim itmX As ListItem
         
   On Error GoTo errGetDataAll
   
   'Clear the old data in the ListView control
   frmMain.ListView1.ListItems.Clear
   frmMain.ListView1.ColumnHeaders.Clear

   'SQLBindColumn on all columns in the result set
   If (Not BindColAll(hStmt)) Then
      GetDataAll = False
      Exit Function
   End If
         
   'Setup the headings on the list view
   For I = 0 To UBound(gsColName)            'Throws exception if # columns = 0
      frmMain.ListView1.ColumnHeaders. _
                  Add , , gsColName(I)       ', ListView1.Width / UBound(gsColName)
   Next I
   ' Set View property to Report.
   frmMain.ListView1.View = lvwReport

   'Retrive all the rows in the Result Set
   I = 0
   Do While (True)
      rc = SQLFetch(hStmt)                            '// fetch one record
      If (rc <> SQL_SUCCESS And rc <> SQL_SUCCESS_WITH_INFO) Then
         '//fall into this only on error or last record
         If (rc = SQL_NO_DATA_FOUND) Then             'if Last record
            S = Str(I) & " rows returned successfully" & Chr(13) & Chr(10)
            'If I >= 1 Then
               'Print out the last row
               'S = S & "Data in last row:" & Chr(13) & Chr(10)
               'J = 0
               'for each column in result set
               'For J = 0 To UBound(gRSData) - 1
               '   K = gRSData(J).lCharLen
               '   S1 = Space(MAXCOLSTRING)
               '   rc = Byte2String(gRSData(J).szCharValue(), S1)
               '   'gsCharValue(J) = S1
               '   S = S & vbTab & "col " & Str(J + 1) & ":" ' %.2d: % 10.10s  %s\r\n",
               '   S = S & vbTab & gsColName(J)
               '   S = S & vbTab & S1 & Chr(13) & Chr(10)
               'Next J
               'MsgBox S, , gAppName
               frmMain.lblNumRows.Caption = S      'return number of rows fetched.
             'End If
         Else            '//must of been error
            '//Let caller retrieve error show_stmt_error(hWnd, hStmt);
            GetDataAll = False
            Exit Function
         End If
         Exit Do
      End If
      
      'Add the fetched record to the listbox
      S1 = Space(MAXCOLSTRING)
      Call Byte2String(gRSData(0).szCharValue(), S1)
      S1 = Left(S1, gRSData(0).lCharLen)
      'Add a new row
      Set itmX = frmMain.ListView1.ListItems. _
                     Add(, , S1)
      'Add each column
      For j = 1 To UBound(gRSData)
         S1 = Space(MAXCOLSTRING)
         'watch for zero length data...
         If gRSData(j).lCharLen > 0 Then
            Call Byte2String(gRSData(j).szCharValue(), S1)
            S1 = Left(S1, gRSData(j).lCharLen)
         Else
            S1 = ""
         End If
         itmX.SubItems(j) = S1
      Next j
      
      I = I + 1                                          '// row counter
   Loop
   
   GetDataAll = True
   Exit Function
   
errGetDataAll:
   'Check for result set of no columns (stored procedure call that
   'did not return a result set correctly, or an error)
   If Err.Number = 9 Then  'UBound() out of range...
      S = "No result set returned"
      frmMain.lblNumRows.Caption = S
      GetDataAll = True
   Else
      S = "Unexpected Error # " & Str(Err.Number) & " was generated by " _
            & Err.Source & Chr(13) & " in Function GetDataAll " & _
            vbCr & Err.Description
      MsgBox S, , gAppName, Err.HelpFile, Err.HelpContext
      GetDataAll = False
   End If

End Function


Sub PrintError(ByVal hdbc As Long, ByVal hStmt As Long)
  ' Retrieve and display an error message for the given connection handle
  ' and statement handle
  Dim rgbValue1 As String
  Dim rgbValue3 As String
  Dim Outlen As Integer
  Dim Native As Long
  Dim rc As Long

  rgbValue1 = Space(6)
  rgbValue3 = Space(148) ', 0)
  
  Do
    rc = SQLError(ghEnv, hdbc, hStmt, _
                  rgbValue1, 0, _
                  rgbValue3, Len(rgbValue3), Outlen)
    Screen.MousePointer = vbArrow
    If rc = SQL_SUCCESS Or rc = SQL_SUCCESS_WITH_INFO Then
        If Outlen = 0 Then
            MsgBox "Error -- No error information available", vbExclamation, gAppName
        Else
            If rc = SQL_ERROR Then
                MsgBox Left$(rgbValue3, Outlen), vbExclamation, gAppName
            Else
                MsgBox Left$(rgbValue3, Outlen), vbExclamation, gAppName
            End If
        End If
    End If
  Loop Until rc <> SQL_SUCCESS

End Sub




